@@ -36,8 +36,8 @@ enum RenameDialog_Option {
     RenameDialog_IsDirectory KIOCORE_ENUMERATOR_DEPRECATED_VERSION_BELATED(5, 82, 5, 78, "Use RenameDialog_DestIsDirectory.") =
         128, ///< @deprecated since 5.78, use RenameDialog_DestIsDirectory instead.
 #endif
-    RenameDialog_DestIsDirectory = 128, ///< @since 5.78. The destination is a directory, the dialog updates labels and tooltips accordingly.
-    RenameDialog_SourceIsDirectory = 256, ///< @since 5.78. The source is a directory, the dialog updates labels and tooltips accordingly.
+    RenameDialog_DestIsDirectory = 128, ///< The destination is a directory, the dialog updates labels and tooltips accordingly. @since 5.78
+    RenameDialog_SourceIsDirectory = 256, ///< The source is a directory, the dialog updates labels and tooltips accordingly. @since 5.78
 };
 /**
  * Stores a combination of #RenameDialog_Option values.
@@ -79,7 +79,7 @@ enum SkipDialog_Option {
     /**
      * Set if the current operation involves copying files/folders with certain
      * characters in their names that are not supported by the destination
-     * filesystem (e.g. VFAT and NTFS disallow "*" in file/folder names).
+     * filesystem (e.g.\ VFAT and NTFS disallow "*" in file/folder names).
      *
      * This will make the SkipDialog show a "Replace" button that can be used
      * to instruct the underlying job to replace any problematic character with
@@ -88,6 +88,17 @@ enum SkipDialog_Option {
      * @since 5.86
      */
     SkipDialog_Replace_Invalid_Chars = 16,
+
+    /**
+     * Set if the current operation @e cannot be retried.
+     *
+     * For example if there is an issue that involves the destination filesystem
+     * support, e.g. VFAT and ExFat don't support symlinks, then retrying doesn't
+     * make sense.
+     *
+     * @since 5.88
+     */
+    SkipDialog_Hide_Retry = 32,
 };
 /**
  * Stores a combination of #SkipDialog_Option values.
