@@ -88,7 +88,7 @@ static bool same_inode(const QT_STATBUF &src, const QT_STATBUF &dest)
 static const QString socketPath()
 {
     const QString runtimeDir = QStandardPaths::writableLocation(QStandardPaths::RuntimeLocation);
-    return QStringLiteral("%1/filehelper%2%3").arg(runtimeDir, KRandom::randomString(6)).arg(getpid());
+    return QStringLiteral("%1/filehelper%2%3").arg(runtimeDir, KRandom::randomString(6)).arg(qlonglong(QThread::currentThreadId()));
 }
 
 static QString actionDetails(ActionType actionType, const QVariantList &args)
@@ -163,11 +163,9 @@ bool FileProtocol::isExtendedACL(acl_t acl)
 }
 #endif
 
-static QHash<KUserId, QString> staticUserCache;
-static QHash<KGroupId, QString> staticGroupCache;
-
 static QString getUserName(KUserId uid)
 {
+    thread_local QHash<KUserId, QString> staticUserCache;
     if (Q_UNLIKELY(!uid.isValid())) {
         return QString();
     }
@@ -185,6 +183,7 @@ static QString getUserName(KUserId uid)
 
 static QString getGroupName(KGroupId gid)
 {
+    thread_local QHash<KGroupId, QString> staticGroupCache;
     if (Q_UNLIKELY(!gid.isValid())) {
         return QString();
     }
@@ -1070,19 +1069,7 @@ void FileProtocol::listDir(const QUrl &url)
         return;
     }
 
-    /* set the current dir to the path to speed up
-       in not having to pass an absolute path.
-       We restore the path later to get out of the
-       path - the kernel wouldn't unmount or delete
-       directories we keep as active directory. And
-       as the slave runs in the background, it's hard
-       to see for the user what the problem would be */
-    const QString pathBuffer(QDir::currentPath());
-    if (!QDir::setCurrent(path)) {
-        closedir(dp);
-        error(ERR_CANNOT_ENTER_DIRECTORY, path);
-        return;
-    }
+    const QByteArray encodedBasePath = _path + '/';
 
     const KIO::StatDetails details = getStatDetails();
     // qDebug() << "========= LIST " << url << "details=" << details << " =========";
@@ -1134,7 +1121,7 @@ void FileProtocol::listDir(const QUrl &url)
             }
             fullPath += filename;
 
-            if (createUDSEntry(filename, QByteArray(ep->d_name), entry, details, fullPath)) {
+            if (createUDSEntry(filename, encodedBasePath + QByteArray(ep->d_name), entry, details, fullPath)) {
 #if HAVE_SYS_XATTR_H
                 if (isNtfsHidden(filename)) {
                     bool ntfsHidden = true;
@@ -1160,9 +1147,6 @@ void FileProtocol::listDir(const QUrl &url)
 
     closedir(dp);
 
-    // Restore the path
-    QDir::setCurrent(pathBuffer);
-
     finished();
 }
 
